﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;

using GermanyRuTest.Common;

namespace GermanyRuTest.ViewModel
{
    /// <summary>
    ///     Class BackgroundUserControlVm.
    ///     ViewModel for BackgroundUserControl
    /// </summary>
    /// <seealso cref="GermanyRuTest.Common.ViewModelBase" />
    /// <seealso cref="GermanyRuTest.BackgroundUserControl" />
    internal class BackgroundUserControlVm : ViewModelBase
    {
        public event EventHandler ExecutionFinished;

        private readonly MediaElement _mediaPlayer;

        private readonly DispatcherTimer _timer = new DispatcherTimer();

        private string _infoText;

        private long _progressPos;

        private string _videoSource;

        private EVisibility _videoVisibility;

        /// <summary>
        ///     Initializes a new instance of the <see cref="BackgroundUserControlVm" /> class.
        /// </summary>
        /// <param name="mediaPlayer">The media player control.</param>
        /// <exception cref="ArgumentNullException">mediaPlayer</exception>
        public BackgroundUserControlVm(MediaElement mediaPlayer)
        {
            if (mediaPlayer == null)
            {
                throw new ArgumentNullException(nameof(mediaPlayer));
            }
            _mediaPlayer = mediaPlayer;
            _mediaPlayer.MediaEnded += MediaPlayerMediaEnded;
            _mediaPlayer.MediaFailed += MediaPlayerMediaFailed;

            _videoSource = "http://hubblesource.stsci.edu/sources/video/clips/details/images/hst_1.mpg";

            _timer.Interval = TimeSpan.FromSeconds(1);
            _timer.Tick += TimerTick;
            _timer.Start();
            ItemsCombiner.Instance.ExecutionStarted += CommandExecutionStarted;

            _videoVisibility = EVisibility.Collapsed;
        }

        /// <summary>
        /// Starts the action.
        /// </summary>
        public void StartAction()
        {
            VideoVisibility = EVisibility.Visible;
            _mediaPlayer.Play();
        }

        /// <summary>
        /// Called when action execution finished.
        /// </summary>
        protected virtual void OnExecutionFinished()
        {
            VideoVisibility = EVisibility.Collapsed;
            ExecutionFinished?.Invoke(this, EventArgs.Empty);
        }

        private void CommandExecutionStarted(object sender, EventArgs e)
        {
            StartAction();
        }

        private void MediaPlayerMediaEnded(object sender, RoutedEventArgs e)
        {
            OnExecutionFinished();
        }

        private void MediaPlayerMediaFailed(object sender, ExceptionRoutedEventArgs e)
        {
            InfoText = e.ErrorException.Message;
        }

        private void TimerTick(object sender, EventArgs e)
        {
            if (_mediaPlayer.Source != null)
            {
                //it will be active when video is loaded
                if (_mediaPlayer.NaturalDuration.HasTimeSpan)
                {
                    double videoTotalMilliseconds = _mediaPlayer.NaturalDuration.TimeSpan.TotalMilliseconds;
                    double pos = _mediaPlayer.Position.TotalMilliseconds * 100 / videoTotalMilliseconds;
                    ProgressPos = (long)pos;

                    InfoText = String.Format(
                        "{0} / {1}",
                        _mediaPlayer.Position.ToString(@"mm\:ss"),
                        _mediaPlayer.NaturalDuration.TimeSpan.ToString(@"mm\:ss"));
                }
                else
                {
                    InfoText = "Loading...";
                }
            }
            else
            {
                InfoText = "No file selected...";
            }
        }

        public string InfoText
        {
            get
            {
                return _infoText;
            }
            set
            {
                _infoText = value;
                OnPropertyChanged(nameof(InfoText));
            }
        }

        public long ProgressPos
        {
            get
            {
                return _progressPos;
            }
            set
            {
                _progressPos = value;
                OnPropertyChanged(nameof(ProgressPos));
            }
        }

        public string VideoSource
        {
            get
            {
                return _videoSource;
            }
            set
            {
                _videoSource = value;
                OnPropertyChanged(nameof(VideoSource));
            }
        }

        public EVisibility VideoVisibility
        {
            get
            {
                return _videoVisibility;
            }
            set
            {
                _videoVisibility = value;
                OnPropertyChanged(nameof(VideoVisibility));
            }
        }
    }
}