﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;

using GermanyRuTest.Common;
using GermanyRuTest.Model;

namespace GermanyRuTest.ViewModel
{
    /// <summary>
    /// Class RulesUserControlVm.
    /// ViewModel for RulesUserControl
    /// </summary>
    /// <seealso cref="GermanyRuTest.Common.ViewModelBase" />
    /// <seealso cref="GermanyRuTest.RulesUserControl" />
    internal class RulesUserControlVm : ViewModelBase
    {
        private const int MaxButtons = 12;

        private readonly List<ButtonUserControlVm> _buttonDataContext = new List<ButtonUserControlVm>();

        private readonly ButtonDataCollection _buttons = new ButtonDataCollection();

        private readonly ValueValidator _valueValidator = new ValueValidator();

        private bool _buttonStartEnable;

        private string _infoValueText;

        private EVisibility _rulesVisibility;

        /// <summary>
        /// Initializes a new instance of the <see cref="RulesUserControlVm"/> class.
        /// </summary>
        public RulesUserControlVm()
        {
            for (int i = MaxButtons - 1; i >= 0; i--)
            {
                _buttons.Add(new ButtonData(i, EButtonState.Normal));
            }
            _buttons.ValueChanged += ButtonsValueChanged;

            foreach (ButtonData buttonData in _buttons)
            {
                ButtonUserControlVm buttonViewModel = new ButtonUserControlVm(buttonData);
                _buttonDataContext.Add(buttonViewModel);
            }
            RulesVisibility = EVisibility.Visible;
            ItemsCombiner.Instance.ExecutionFinished += NotificationExecutionFinished;
        }

        /// <summary>
        /// Starts the external action.
        /// </summary>
        public void StartAction()
        {
            RulesVisibility = EVisibility.Collapsed;
            ItemsCombiner.Instance.ActionExecutionStart();
        }

        private void ButtonsValueChanged(object sender, PropertyChangedEventArgs e)
        {
            InfoValueText = String.Format("0x{0:X3}", _buttons.SummaryValue);
            ButtonStartEnable = _valueValidator.IsValid(_buttons.SummaryValue);
        }

        private void NotificationExecutionFinished(object sender, EventArgs e)
        {
            RulesVisibility = EVisibility.Visible;
        }

        /// <summary>
        /// Gets the buttons data context.
        /// </summary>
        /// <value>The buttons data context.</value>
        public ReadOnlyCollection<ButtonUserControlVm> ButtonsDataContext
        {
            get
            {
                return _buttonDataContext.AsReadOnly();
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether button start enable.
        /// </summary>
        /// <value><c>true</c> if button start enable; otherwise, <c>false</c>.</value>
        public bool ButtonStartEnable
        {
            get
            {
                return _buttonStartEnable;
            }
            set
            {
                _buttonStartEnable = value;
                OnPropertyChanged(nameof(ButtonStartEnable));
            }
        }

        /// <summary>
        /// Gets the information text.
        /// </summary>
        /// <value>The information text.</value>
        public string InfoText
        {
            get
            {
                return String.Format(
                    "Select right buttons combination (0x{0}) to start movie.",
                    ValueValidator.ValidValue.ToString("X3"));
            }
        }

        /// <summary>
        /// Gets or sets the buttons summary value as text.
        /// </summary>
        /// <value>The information value text.</value>
        public string InfoValueText
        {
            get
            {
                return _infoValueText;
            }
            set
            {
                _infoValueText = value;
                OnPropertyChanged(nameof(InfoValueText));
            }
        }

        /// <summary>
        /// Gets or sets the rules panel visibility.
        /// </summary>
        /// <value>The rules visibility.</value>
        public EVisibility RulesVisibility
        {
            get
            {
                return _rulesVisibility;
            }
            set
            {
                _rulesVisibility = value;
                OnPropertyChanged(nameof(RulesVisibility));
            }
        }
    }
}